from zope.app import zapi
from zope.app.component import getNextUtility
from zope.app.component.interfaces import ISite
from zope.app.component.site import LocalSiteManager
from zope.app.security.interfaces import IAuthentication
from zope.app.security.interfaces import IAuthenticatedGroup, IEveryoneGroup
from zope.component import getUtility
from zope.security.checker import ProxyFactory

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.security import Principal
from schooltool.app.security import PersonContainerAuthenticationPlugin
from schooltool.person.interfaces import IPersonFactory

class LDAPAuthenticationPlugin(PersonContainerAuthenticationPlugin):
    """
    A local utility to authenticate with LDAP
    """

    def __init__(self):
        super(LDAPAuthenticationPlugin, self).__init__()
        from ldappas.authentication import LDAPAuthentication
        self.auth = LDAPAuthentication()
        self.auth.adapterName = 'SchoolToolLDAP'
        self.auth.searchScope = 'sub'
        self.auth.searchBase = 'dc=sociality,dc=org'
        self.auth.groupsSearchBase = 'ou=groups'
        self.auth.groupsSearchScope = 'sub'
        self.auth.loginAttribute = 'uid'
        self.auth.principalIdPrefix = ''
        self.auth.idAttribute = 'uid'
        self.auth.titleAttribute = 'cn'
        self.auth.groupsAttribute = 'apple-group'
        self.auth.groupIdAttribute = 'apple-group-realname'

    def getLDAPAdapter(self):
        return self.auth.getLDAPAdapter()

    def _checkPassword(self, username, password):
        return self.auth.authenticateCredentials({'login': username or '',
                                                  'password': password or ''})

    def getPrincipal(self, id):
        """Get principal meta-data.

        Returns principals for groups and persons.
        """
        app = ISchoolToolApplication(None)
        #app = {'persons':{}, 'groups':{}}
        if id.startswith(self.person_prefix):
            username = id[len(self.person_prefix):]
            p_info = self.auth.principalInfo(username)

            if p_info:
                if username in app['persons']:
                    person = app['persons'][username]
                else:
                    factory = getUtility(IPersonFactory)
                    person = factory(p_info.id, p_info.title)
                    app['persons'][username] = person
                principal = Principal(id, person.title,
                                      person=ProxyFactory(person))
                for group in person.groups:
                    group_principal_id = self.group_prefix + group.__name__
                    principal.groups.append(group_principal_id)
                authenticated = zapi.queryUtility(IAuthenticatedGroup)
                if authenticated:
                    principal.groups.append(authenticated.id)
                everyone = zapi.queryUtility(IEveryoneGroup)
                if everyone:
                    principal.groups.append(everyone.id)
                return principal

        if id.startswith(self.group_prefix):
            group_name = id[len(self.group_prefix):]
            if group_name in app['groups']:
                group = app['groups'][group_name]
                # Group membership is not supported in SB, so we don't bother
                # filling in principal.groups.
                return Principal(id, group.title)
        return None


from config import LDAP_HOST, LDAP_PORT, BIND_DN, BIND_PASSWORD
from ldapadapter.utility import LDAPAdapter
ldapAdapter = LDAPAdapter(LDAP_HOST, LDAP_PORT,
                          bindDN=BIND_DN, bindPassword=BIND_PASSWORD)


def setupLDAPAdapter(site):
    from ldapadapter.interfaces import ILDAPAdapter
    from ldapadapter.utility import LDAPAdapter
    from config import LDAP_HOST, LDAP_PORT, BIND_DN, BIND_PASSWORD
    if not ISite.providedBy(site):
        site.setSiteManager(LocalSiteManager(site))

    manager = site.getSiteManager()
    ldapAd = LDAPAdapter(LDAP_HOST, LDAP_PORT,
                         bindDN=BIND_DN, bindPassword=BIND_PASSWORD)
    from zope.component import provideUtility
    provideUtility(ldapAd, ILDAPAdapter, 'SchoolToolLDAP')
